package com.goodbackend;

import com.goodbackend.notify.PgCallback;
import com.goodbackend.notify.PgNotifyService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.context.event.ApplicationReadyEvent;
import org.springframework.context.event.EventListener;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;

/**
 * Example of using notification service
 *
 * @author Roman Bouchner roman.bouchner@goodbackend.com
 */
@SpringBootApplication
public class ExampleNotify {

	@Autowired
	private PgNotifyService pgNotifyService;

	private final String EVENT_NAME = "myEvent";

	private final PgCallback eventListener = (eventParameter) -> {
		//this code can be run in different java backend, different machine over network
		System.out.println("Event received with parameter: " + eventParameter);
	};

	@EventListener(ApplicationReadyEvent.class)
	public void start() {
		//send event notification. All java backend connected to the same database will receive this notification
		pgNotifyService.notify("myEvent", "myEventParameter");
		System.out.println("Event sent..");
	}

	@PostConstruct
	private void init() {
		//register listener
		pgNotifyService.addServiceListener(EVENT_NAME,eventListener);
	}

	@PreDestroy
	private void destroy() {
		//unregister listener
		pgNotifyService.removeServiceListener(eventListener);
	}

	public static void main(final String[] args) {
		SpringApplication.run(ExampleNotify.class, args);
	}

}
